use crate::app_interface::{Party, PartyImpl};
use crate::handling::cached_value::CachedValue;
use crate::handling::Cacheable;
use crate::{cached_value, cached_value_getters, ExternRef, ExternalReferenceType};
use alloc::rc::Rc;

pub trait BattlePartyTrait {
    fn party(&self) -> Party;
}

pub type BattleParty = Rc<dyn BattlePartyTrait>;

struct BattlePartyInner {
    reference: ExternRef<BattlePartyImpl>,
    party: CachedValue<Rc<PartyImpl>>,
}

#[derive(Clone)]
pub struct BattlePartyImpl {
    inner: Rc<BattlePartyInner>,
}

#[cfg(not(feature = "mock_data"))]
impl BattlePartyImpl {
    pub fn new(reference: ExternRef<BattlePartyImpl>) -> Self {
        Self::from_ref(reference, &|reference| Self {
            inner: Rc::new(BattlePartyInner {
                reference,
                party: cached_value!({
                    Rc::new(battle_party_get_party(reference).get_value().unwrap())
                }),
            }),
        })
    }
}

#[cfg(not(feature = "mock_data"))]
impl BattlePartyTrait for BattlePartyImpl {
    cached_value_getters! {
        fn party(&self) -> Party;
    }
}

crate::handling::cacheable::cacheable!(BattlePartyImpl);

#[cfg(not(feature = "mock_data"))]
impl ExternalReferenceType for BattlePartyImpl {
    fn from_extern_value(reference: ExternRef<Self>) -> Self {
        Self::new(reference)
    }
}

#[cfg(not(feature = "mock_data"))]
extern "wasm" {
    fn battle_party_get_party(r: ExternRef<BattlePartyImpl>) -> ExternRef<PartyImpl>;
}
