use crate::handling::cacheable::Cacheable;
use crate::handling::cached_value::CachedValue;
use crate::{
    cached_value, cached_value_getters, EffectParameter, ExternRef, ExternalReferenceType,
    ImmutableList, StringKey, VecExternRef,
};
use alloc::rc::Rc;

struct AbilityInner {
    reference: ExternRef<AbilityImpl>,
    name: CachedValue<StringKey>,
    effect: CachedValue<StringKey>,
    parameters: CachedValue<ImmutableList<EffectParameter>>,
}

#[cfg_attr(test, mockall::automock)]
pub trait Ability {
    fn name(&self) -> StringKey;
    fn effect(&self) -> StringKey;
}

#[derive(Clone)]
pub struct AbilityImpl {
    inner: Rc<AbilityInner>,
}

impl AbilityImpl {
    #[cfg(not(feature = "mock_data"))]
    pub fn new(reference: ExternRef<Self>) -> Self {
        Self::from_ref(reference, &|reference| Self {
            inner: Rc::new(AbilityInner {
                reference,
                name: cached_value!({ ability_get_name(reference).get_value().unwrap() }),
                effect: cached_value!({ ability_get_effect(reference).get_value().unwrap() }),
                parameters: cached_value!({
                    ability_get_parameters(reference).get_immutable_list()
                }),
            }),
        })
    }
}

impl Ability for AbilityImpl {
    cached_value_getters! {
        fn name(&self) -> StringKey;
        fn effect(&self) -> StringKey;
    }
}

#[cfg(not(feature = "mock_data"))]
impl ExternalReferenceType for AbilityImpl {
    fn from_extern_value(reference: ExternRef<Self>) -> Self {
        Self::new(reference)
    }
}

crate::handling::cacheable::cacheable!(AbilityImpl);

#[derive(Copy, Clone, Debug, Eq, PartialEq)]
#[repr(C)]
pub struct AbilityIndex {
    pub hidden: bool,
    pub index: u8,
}

#[cfg(not(feature = "mock_data"))]
extern "wasm" {
    fn ability_get_name(r: ExternRef<AbilityImpl>) -> ExternRef<StringKey>;
    fn ability_get_effect(r: ExternRef<AbilityImpl>) -> ExternRef<StringKey>;
    fn ability_get_parameters(r: ExternRef<AbilityImpl>) -> VecExternRef<EffectParameter>;
}
